#define _CRT_SECURE_NO_DEPRECATE 1 
#include <stdio.h>
#include <stdlib.h>
#include <direct.h>
#include <memory.h>
#include <string.h>
#include <math.h>
#include "rand.h"
#include "main.h"
#include "global.h"
#include "de.h"
#include "function.h"

// Important paramaters for running DE
int     DEvariant;  // choice parameter for screen output
double  F, CR;      // control variables of DE
int     NP;         // Population size or the no. of individuals

//====================================================
//   Essential data used by standard DE
//====================================================
int    imin;         // index to member with the best cost (obj. func. val.)
double mincost;      // obj. func. val. of the best
double cost[MAXPOP]; // objective function values (f.)
double trial_cost;   // f. of the trial vector
double best[MAXDIM], bestit[MAXDIM];
double *curpop[MAXPOP], *nxtpop[MAXPOP], *pswap;

//====================================================
//   Code definitions begin here
//====================================================

void allocate_de_memory()
{
	int	n;

	for ( n = 0; n < NP; n++ ) {
		curpop[n] = (double *)calloc(D, sizeof(double));
		nxtpop[n] = (double *)calloc(D, sizeof(double));
	}
}

void deallocate_de_memory()
{
	int n;

    for ( n = 0; n < NP; n++ ) {
		free(curpop[n]); free(nxtpop[n]);
	}
}

void reset_de()
{
	int n, d;

    // ˹Ъҡ
    for ( n = 0; n < NP; n++ ) {
        for ( d = 0; d < D; d++ )
            curpop[n][d] = randreal(low[d], hi[d]);
        cost[n] = DEevaluate(curpop[n]);    //¡ DEevaluate() ͤӹǳ cost value
    }
    // Find the best individual at startup
    imin = 0;
    mincost = cost[0];
    for ( n = 1; n < NP; n++ )   //  ӡ纤 mincost 
       if ( cost[n] < mincost ) {
	       mincost = cost[n];
	       imin = n;
       }
	dcopy(D, best,   curpop[imin]);  // save best member ever
    dcopy(D, bestit, curpop[imin]);  // save best member of generation
}

//=========================================================
//   Main entrance of the standard DE
//=========================================================
double de()      // Ӣ鹵͹ 2 Mutation
{
	int	target, d, i; //, cf;

	reset_de();     //  ˹Ъҡ
	CR = 0.9;		// ˹ Crossover constant 
	F = 0.5;		// ˹ Mutation constant

	nfc = 0;
	logc = 0;
	for ( gen = 1 ; nfc <= maxnfc ; gen++ ) {      // Gen  1 ӨҨФú Generation ˹  2000
        imin = 0;
		for ( target = 0 ; target < NP; target++ ) {     //  ˹ Target vector  NP  0   Target á

			differential_mutation(target); // ӡա 3 vector

            // Trial mutation now in x[]. Check its boundaries
			for ( d = 0; d < D; d++) {
				if ( x[d] < low[d] )
					x[d] = randreal ( low[d], 0.005*(hi[d]-low[d]) );
				else if ( x[d] > hi[d] ) {
					double space = 0.005 * ( hi[d] - low[d] );
					x[d] = randreal( hi[d] - space, space );
				}
			}

            // Ҥ cost value ͧ trial vector
            trial_cost = DEevaluate(x);

			//if ( nfc % logstep2screen == 0 )
			//	change_parameters();

			// ºº cost value ͧ trial vector   target vector   trial ¡ target 
			// ᷹ trial   target 蹵
            if (trial_cost <= cost[target]) { // Is objective function improved?
	            cost[target] = trial_cost;
	            dcopy(D, nxtpop[target], x);
                if ( trial_cost < mincost ) { // Was this a new minimum?
	                mincost = trial_cost;     // If so, set mincost to new low.
	                imin = target;
	                dcopy(D, best, x);
	            }
            } else {  // Not improved, then replace target with old v.
                dcopy(D, nxtpop[target], curpop[target]);
            }
        }

        dcopy(D, bestit, best); // Save best member of the current gen.
		for ( i = 0 ; i < NP ; i++ ) {
			pswap = curpop[i];
			curpop[i] = nxtpop[i];
			nxtpop[i] = pswap; // swap pop. for next gen.
		}
	}
	putchar('\n');
    return mincost;
}

void differential_mutation(int target)
{
    int n, L;
    int r1, r2, r3;      //  3 vector

    do r1 = (int)(unirandom()*NP); while (r1==target);  // vector  r1 µͧӡѺ Target vector
    do r2 = (int)(unirandom()*NP); while (r2==target || r2==r1); // vector  r2 µͧӡѺ Target vector ӡѺ r1
    do r3 = (int)(unirandom()*NP); while (r3==target || r3==r1 || r3==r2);// vector  r2 µͧӡѺ Target ӡѺ r1  r2

    /*-------DE/rand/1/bin--------------------------------*/
	
        dcopy(D, x, curpop[target]);
        n = (int)( unirandom() * D );
		for ( L = 0 ; L < D ; L++ ) { /* perform D binomial trials */
            if ( unirandom() < CR || 
				L == D-1 ) /* change at least 1 parameter */
                //x[n] = (*pold)[r1][n] + F*( (*pold)[r2][n] - (*pold)[r3][n] );
				x[n] = curpop[r1][n] + F*( curpop[r2][n] - curpop[r3][n] );
            n = ( n + 1 ) % D;
        }
}

double DEevaluate(double x[])
{
	double ret = Sphere(x);

    nfc++;
    // Log to file
	if ( nfc % logstep2file == 0 || nfc == 500 ) {
        lognfc[logc]  = nfc;
        logval[logc] += mincost;
        logc++;
    }
	// Onscreen logging
	if ( nfc % logstep2screen == 0 || nfc == 500 ) {
		printf("%s\t%s\tR%02d\t%d\t%d\t%10.04g\n", 
			stratename, functName, nrun, gen, nfc, mincost);
		fflush(stdout);
	}
	return ret;
}

void dcopy(int D, double dest[], double source[])
{
   register int j;
   for (j=0; j<D; j++)
      dest[j] = source[j];
}