/* Definition of random number generation routines */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rand.h"

/* Variable declarations for the random number generator */
double seed;
double oldrand[55];
int    jrand;
int    rndcalcflag;
double rndx1, rndx2;
long   lseed = -1234;

/* Get seed number for random and start it up */
void randomize()
{
    int j1;
    for ( j1 = 0 ; j1 <= 54 ; j1++ )
        oldrand[j1] = 0.0;
    jrand = 0;
    warmup_random(seed);
    return;
}

/* Get randomize off and running */
void warmup_random (double seed)
{
    int j1, ii;
    double new_random, prev_random;
    oldrand[54] = seed;
    new_random = 0.000000001;
    prev_random = seed;
    for( j1 = 1 ; j1 <= 54 ; j1++ ) {
        ii = (21*j1)%54;
        oldrand[ii] = new_random;
        new_random = prev_random-new_random;
        if(new_random<0.0)
            new_random += 1.0;

        prev_random = oldrand[ii];
    }
    advance_random ();
    advance_random ();
    advance_random ();
    jrand = 0;
    return;
}

/* Create next batch of 55 random numbers */
void advance_random ()
{
    int j1;
    double new_random;
    for(j1=0; j1<24; j1++)
    {
        new_random = oldrand[j1]-oldrand[j1+31];
        if(new_random<0.0)
            new_random = new_random+1.0;

        oldrand[j1] = new_random;
    }
    for(j1=24; j1<55; j1++)
    {
        new_random = oldrand[j1]-oldrand[j1-24];
        if(new_random<0.0)
            new_random = new_random+1.0;

        oldrand[j1] = new_random;
    }
}

/* Fetch a single random number between 0.0 and 1.0 */
double randomperc()
{
    jrand++;
    if ( jrand >= 55 ) {
        jrand = 1;
        advance_random();
    }
    return (double)oldrand[jrand];
}

/* Fetch a single random integer between low and high including the bounds */
int randint(int low, int high)
{
    int res;
    if (low >= high)
        res = low;
    else {
        res = low + (int)(randomperc()*(high-low+1));
        if (res > high) res = high;
    }
    return res;
}

/* Fetch a single random real number between low and high including the bounds */
double randreal(double low, double high)
{
    return (low + (high-low)*randomperc());
}

/* initialize_pso the randome generator for normal distribution */
void initrandomnormaldeviate()
{
    rndcalcflag = 1;
    return;
}

/* Return the noise value */
double noise(double mu, double sigma)
{
    return (randomnormaldeviate()*sigma) + mu;
}

/* Compute the noise */
double randomnormaldeviate()
{
    double t;
    if(rndcalcflag) {
        rndx1 = sqrt(- 2.0*log(randomperc()));
        t = 6.2831853072*randomperc();
        rndx2 = sin(t);
        rndcalcflag = 0;
        return(rndx1*cos(t));
    } else {
        rndcalcflag = 1;
        return(rndx1*rndx2);
    }
}

/*------Constants for uniform_random()--------------------------------------------*/

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.0e-14
#define RNMX (1.0-EPS)

double uniform_random(long *idum)
/**C*F****************************************************************
** SRC-FUNCTION :uniform_random()                                   **
** LONG_NAME    :random_uniform                                     **
** DESCRIPTION  :uniform_random() generates an equally distributed  **
**               random number in the interval [0,1]. For further   **
**               reference see Press, W.H. et alii, Numerical       **
**               Recipes in C, Cambridge University Press, 1992.    **
** PARAMETERS     :*idum    serves as a seed value                  **
** PRECONDITIONS  :*idum must be negative on the first call.        **
** POSTCONDITIONS :*idum will be changed                            **
***C*F*E*************************************************************/
{
    long j, k;
    static long idum2 = 123456789;
    static long iy = 0;
    static long iv[NTAB];
    double tempCR;

    if (*idum <= 0) {
        if (-(*idum) < 1) *idum=1;
        else *idum = -(*idum);
        idum2=(*idum);
        for (j=NTAB+7; j>=0; j--) {
            k=(*idum)/IQ1;
            *idum=IA1*(*idum-k*IQ1)-k*IR1;
            if (*idum < 0) *idum += IM1;
            if (j < NTAB) iv[j] = *idum;
        }
        iy=iv[0];
    }
    k=(*idum)/IQ1;
    *idum=IA1*(*idum-k*IQ1)-k*IR1;
    if (*idum < 0) *idum += IM1;
    k=idum2/IQ2;
    idum2=IA2*(idum2-k*IQ2)-k*IR2;
    if (idum2 < 0) idum2 += IM2;
    j=iy/NDIV;
    iy=iv[j]-idum2;
    iv[j] = *idum;
    if (iy < 1) iy += IMM1;
    if ((tempCR=AM*iy) > RNMX) return RNMX; else return tempCR;
}